#######################
# Built-in namespaces #
#######################

declare namespace xsd      = "http://www.w3.org/2001/XMLSchema-datatypes"
declare namespace xs       = "http://www.w3.org/2001/XMLSchema"
declare namespace xsi      = "http://www.w3.org/2001/XMLSchema-instance"
declare namespace fn       = "http://www.w3.org/2002/11/xquery-functions"
declare namespace op       = "http://www.w3.org/2002/11/xquery-operators"
declare namespace xq       = "http://www.w3.org/2002/08/xquery"
declare namespace glx      = "http://db.bell-labs.com/galax"


#############################
# XML Schema wildcard types #
#############################

define type xs:AnyNumeric      { xsd:decimal | xsd:float | xsd:double | xsd:integer | xsd:int } 
 
define type xs:AnyScalar       { xsd:string |
			 	 xsd:boolean |
			 	 xsd:duration |
			 	 xsd:dateTime |
			 	 xsd:time |
			 	 xsd:date |
			 	 xsd:gYearMonth |
			 	 xsd:gYear |
			 	 xsd:gMonthDay |
			 	 xsd:gDay |
			 	 xsd:gMonth |
			 	 xsd:hexBinary |
			 	 xsd:base64Binary |
			 	 xsd:anyURI |
			 	 xsd:QName |
			 	 xsd:NOTATION |
			 	 type xs:AnyNumeric |
	                      	 xsd:untyped }

define type xs:AnySimpleType   { type xs:AnyScalar* }
define type xs:AnyAttribute    { attribute * { type xs:AnySimpleType } }
define type xs:AnyElement      { element * { type xs:AnyContentModel } }
define type xs:AnyContentModel { (type xs:AnyAttribute*, (type xs:AnySimpleType | (type xs:AnyElement | text)*)) }
define type xs:AnyDocument     { document { type xs:AnyNode* } }


#########################
# XQuery Wildcard types #
#########################

define type xs:AnyNode        { type xs:AnyDocument | type xs:AnyElement | type xs:AnyAttribute | text }
define type xs:AnyItem        { type xs:AnyNode | type xs:AnyScalar }
define type xs:AnyType        { type xs:AnyItem* }


#################
# F&O functions #
#################

#
# F&O Section 2. Accessors
#

define function signature  fn:node-kind(node)         as (xsd:string)
define function signature  fn:node-name(node)         as (xsd:QName?)
define function signature  fn:string(item?)           as (xsd:string)
define function signature  fn:data(item*)             as (atomic*) 
define function signature  fn:base-uri()              as (xsd:anyURI)
define function signature  fn:base-uri(node)          as (xsd:anyURI?)


#
# F&O Section 5. Functions and Operators on Numbers
#

define function signature  op:numeric-add(numeric?, numeric?)      as (numeric?)
define function signature  op:numeric-subtract(numeric?, numeric?) as (numeric?)
define function signature  op:numeric-multiply(numeric?, numeric?) as (numeric?)
define function signature  op:numeric-divide(numeric?, numeric?)   as (numeric?)
define function signature  op:numeric-idivide(numeric?, numeric?)  as (numeric?)
define function signature  op:numeric-mod(numeric?, numeric?)      as (numeric?)

define function signature  op:numeric-unary-plus(numeric?)        as (numeric?)
define function signature  op:numeric-unary-minus(numeric?)       as (numeric?)

# Note: arithmetic functions are monomorphic in Galax for now...
# xsd:int
define function signature  op:int-add(xsd:int?, xsd:int?)      	 as (xsd:int?)
define function signature  op:int-subtract(xsd:int?, xsd:int?) 	 as (xsd:int?)
define function signature  op:int-multiply(xsd:int?, xsd:int?) 	 as (xsd:int?)
define function signature  op:int-divide(xsd:int?, xsd:int?)   	 as (xsd:int?)
define function signature  op:int-mod(xsd:int?, xsd:int?)      	 as (xsd:int?)

define function signature  op:int-unary-plus(xsd:int?)    	 as (xsd:int?)
define function signature  op:int-unary-minus(xsd:int?)   	 as (xsd:int?)

define function signature  op:int-equal(xsd:int, xsd:int)        as (xsd:boolean)
define function signature  op:int-nequal(xsd:int, xsd:int)       as (xsd:boolean)
define function signature  op:int-lt(xsd:int, xsd:int)           as (xsd:boolean)
define function signature  op:int-gt(xsd:int, xsd:int)           as (xsd:boolean)
define function signature  op:int-le(xsd:int, xsd:int)           as (xsd:boolean)
define function signature  op:int-ge(xsd:int, xsd:int)           as (xsd:boolean)

# xsd:integer
define function signature  op:integer-add(xsd:integer?, xsd:integer?)      as (xsd:integer?)
define function signature  op:integer-subtract(xsd:integer?, xsd:integer?) as (xsd:integer?)
define function signature  op:integer-multiply(xsd:integer?, xsd:integer?) as (xsd:integer?)
define function signature  op:integer-divide(xsd:integer?, xsd:integer?)   as (xsd:integer?)
define function signature  op:integer-mod(xsd:integer?, xsd:integer?)      as (xsd:integer?)

define function signature  op:integer-unary-plus(xsd:integer?)    	 as (xsd:integer?)
define function signature  op:integer-unary-minus(xsd:integer?)   	 as (xsd:integer?)

define function signature  op:integer-equal(xsd:integer, xsd:integer) as (xsd:boolean)
define function signature  op:integer-nequal(xsd:integer, xsd:integer) as (xsd:boolean)
define function signature  op:integer-lt(xsd:integer, xsd:integer)    as (xsd:boolean)
define function signature  op:integer-gt(xsd:integer, xsd:integer)    as (xsd:boolean)
define function signature  op:integer-le(xsd:integer, xsd:integer)    as (xsd:boolean)
define function signature  op:integer-ge(xsd:integer, xsd:integer)    as (xsd:boolean)

# xsd:decimal
define function signature  op:decimal-add(xsd:decimal?, xsd:decimal?)      as (xsd:decimal?)
define function signature  op:decimal-subtract(xsd:decimal?, xsd:decimal?) as (xsd:decimal?)
define function signature  op:decimal-multiply(xsd:decimal?, xsd:decimal?) as (xsd:decimal?)
define function signature  op:decimal-divide(xsd:decimal?, xsd:decimal?)   as (xsd:decimal?)
define function signature  op:decimal-mod(xsd:decimal?, xsd:decimal?)      as (xsd:decimal?)

define function signature  op:decimal-unary-plus(xsd:decimal?)         as (xsd:decimal?)
define function signature  op:decimal-unary-minus(xsd:decimal?)        as (xsd:decimal?)

define function signature  op:decimal-equal(xsd:decimal, xsd:decimal) as (xsd:boolean)
define function signature  op:decimal-nequal(xsd:decimal, xsd:decimal) as (xsd:boolean)
define function signature  op:decimal-lt(xsd:decimal, xsd:decimal)    as (xsd:boolean)
define function signature  op:decimal-gt(xsd:decimal, xsd:decimal)    as (xsd:boolean)
define function signature  op:decimal-le(xsd:decimal, xsd:decimal)    as (xsd:boolean)
define function signature  op:decimal-ge(xsd:decimal, xsd:decimal)    as (xsd:boolean)

# xsd:float
define function signature  op:float-add(xsd:float?, xsd:float?)         as (xsd:float?)
define function signature  op:float-subtract(xsd:float?, xsd:float?)    as (xsd:float?)
define function signature  op:float-multiply(xsd:float?, xsd:float?)    as (xsd:float?)
define function signature  op:float-divide(xsd:float?, xsd:float?)      as (xsd:float?)
define function signature  op:float-mod(xsd:float?, xsd:float?)         as (xsd:float?)

define function signature  op:float-unary-plus(xsd:float?)    	      as (xsd:float?)
define function signature  op:float-unary-minus(xsd:float?)   	      as (xsd:float?)

define function signature  op:float-equal(xsd:float, xsd:float)       as (xsd:boolean)
define function signature  op:float-nequal(xsd:float, xsd:float)      as (xsd:boolean)
define function signature  op:float-lt(xsd:float, xsd:float)          as (xsd:boolean)
define function signature  op:float-gt(xsd:float, xsd:float)          as (xsd:boolean)
define function signature  op:float-le(xsd:float, xsd:float)          as (xsd:boolean)
define function signature  op:float-ge(xsd:float, xsd:float)          as (xsd:boolean)

# xsd:double
define function signature  op:double-add(xsd:double?, xsd:double?)      as (xsd:double?)
define function signature  op:double-subtract(xsd:double?, xsd:double?) as (xsd:double?)
define function signature  op:double-multiply(xsd:double?, xsd:double?) as (xsd:double?)
define function signature  op:double-divide(xsd:double?, xsd:double?)   as (xsd:double?)
define function signature  op:double-mod(xsd:double?, xsd:double?)      as (xsd:double?)

define function signature  op:double-unary-plus(xsd:double?)    	as (xsd:double?)
define function signature  op:double-unary-minus(xsd:double?)   	as (xsd:double?)
       
define function signature  op:double-equal(xsd:double, xsd:double)    as (xsd:boolean)
define function signature  op:double-nequal(xsd:double, xsd:double)   as (xsd:boolean)
define function signature  op:double-lt(xsd:double, xsd:double)       as (xsd:boolean)
define function signature  op:double-gt(xsd:double, xsd:double)       as (xsd:boolean)
define function signature  op:double-le(xsd:double, xsd:double)       as (xsd:boolean)
define function signature  op:double-ge(xsd:double, xsd:double)       as (xsd:boolean)

define function signature  fn:floor(xsd:double)		       	      as (xsd:double)
define function signature  fn:ceiling(xsd:double)              	      as (xsd:double)
define function signature  fn:round(xsd:double)		       	      as (xsd:double)

#
# F&O Section 6. Functions on Strings
#

# Note: This part of the F&O document has not been revised, and is
# probably sloppy

# fn:concat may be variadic
define function signature  fn:concat(xsd:string, xsd:string)                      as (xsd:string)
define function signature  fn:string-join(xsd:string*, xsd:string?)               as (xsd:string)
define function signature  fn:starts-with(xsd:string, xsd:string)                 as (xsd:boolean)
define function signature  fn:ends-with(xsd:string, xsd:string)                   as (xsd:boolean)
define function signature  fn:contains(xsd:string, xsd:string)                    as (xsd:boolean)
define function signature  fn:substring(xsd:string, xsd:int, xsd:int)             as (xsd:string)
define function signature  fn:string-length(xsd:string)                           as (xsd:int)
define function signature  fn:substring-before(xsd:string, xsd:string)            as (xsd:string)
define function signature  fn:substring-after(xsd:string, xsd:string)             as (xsd:string)
define function signature  fn:normalize-space(xsd:string)                         as (xsd:string)
define function signature  fn:upper-case(xsd:string)                              as (xsd:string)
define function signature  fn:lower-case(xsd:string)                              as (xsd:string)
define function signature  fn:translate(xsd:string, xsd:string, xsd:string)       as (xsd:string)
define function signature  fn:string-pad(xsd:string, xsd:int, xsd:string)         as (xsd:string)

define function signature  op:string-equal(xsd:string, xsd:string)     as (xsd:boolean)
define function signature  op:string-nequal(xsd:string, xsd:string)    as (xsd:boolean)
define function signature  op:string-lt(xsd:string, xsd:string)        as (xsd:boolean)
define function signature  op:string-gt(xsd:string, xsd:string)        as (xsd:boolean)
define function signature  op:string-le(xsd:string, xsd:string)        as (xsd:boolean)
define function signature  op:string-ge(xsd:string, xsd:string)        as (xsd:boolean)


#
# F&O Section 7. Functions and Operators on Booleans
#

define function signature  fn:true()			      as (xsd:boolean)
define function signature  fn:false()			      as (xsd:boolean)

define function signature  op:or(xsd:boolean, xsd:boolean)    as (xsd:boolean)
define function signature  op:and(xsd:boolean, xsd:boolean)   as (xsd:boolean)
define function signature  fn:not(xsd:boolean)		      as (xsd:boolean)

define function signature  op:boolean-equal(xsd:boolean, xsd:boolean) as (xsd:boolean)
define function signature  op:boolean-nequal(xsd:boolean, xsd:boolean) as (xsd:boolean)
define function signature  op:boolean-lt(xsd:boolean, xsd:boolean)    as (xsd:boolean)
define function signature  op:boolean-gt(xsd:boolean, xsd:boolean)    as (xsd:boolean)
define function signature  op:boolean-le(xsd:boolean, xsd:boolean)    as (xsd:boolean)
define function signature  op:boolean-ge(xsd:boolean, xsd:boolean)    as (xsd:boolean)

#
# F&O Section 8. Functions and Operators on Durations, Dates, and Times
#
# define function signature  op:duration-equal(xsd:duration,xsd:duration) as (xsd:boolean)
# define function signature  op:yearMonthDuration-equal(xsd:duration,xsd:duration) as (xsd:boolean)
# define function signature  op:yearMonthDuration-lt(xsd:duration,xsd:duration) as (xsd:boolean)
# define function signature  op:yearMonthDuration-greater-than(xsd:duration,xsd:duration) as (xsd:boolean)
# define function signature  op:dayTimeDuration-equal(xsd:duration,xsd:duration) as (xsd:boolean)
# define function signature  op:dayTimeDuration-lt(xsd:duration,xsd:duration) as (xsd:boolean)
# define function signature  op:dayTimeDuration-greater-than(xsd:duration,xsd:duration) as (xsd:boolean)
# define function signature  op:dateTime-equal(xsd:duration,xsd:duration) as (xsd:boolean)
# define function signature  op:dateTime-lt(xsd:duration,xsd:duration) as (xsd:boolean)
# define function signature  op:dateTime-greater-than(xsd:duration,xsd:duration) as (xsd:boolean)
# define function signature  op:date-equal(xsd:duration,xsd:duration) as (xsd:boolean)
# define function signature  op:date-lt(xsd:duration,xsd:duration) as (xsd:boolean)
# define function signature  op:date-greater-than(xsd:duration,xsd:duration) as (xsd:boolean)
# define function signature  op:time-equal(xsd:duration,xsd:duration) as (xsd:boolean)
# define function signature  op:time-lt(xsd:duration,xsd:duration) as (xsd:boolean)
# define function signature  op:time-greater-than(xsd:duration,xsd:duration) as (xsd:boolean)
# define function signature  op:gYearMonth-equal(xsd:duration,xsd:duration) as (xsd:boolean)
# define function signature  op:gYear-equal(xsd:duration,xsd:duration) as (xsd:boolean)
# define function signature  op:gMonthDay-equal(xsd:duration,xsd:duration) as (xsd:boolean)
# define function signature  op:gMonth-equal(xsd:duration,xsd:duration) as (xsd:boolean)
# define function signature  op:gDay-equal(xsd:duration,xsd:duration) as (xsd:boolean)


#
# F&O Section 9. Functions on QNames
#

define function signature  fn:expanded-QName(xsd:string,xsd:string) as (xsd:QName)
define function signature  fn:get-local-name-from-QName(xsd:QName)  as (xsd:string?)
define function signature  fn:get-namespace-from-QName(xsd:QName?)  as (xsd:anyURI?)
define function signature  op:QName-equal(xsd:QName, xsd:QName)     as (xsd:boolean)
define function signature  op:QName-nequal(xsd:QName, xsd:QName)    as (xsd:boolean)


#
# F&O Section 10. Functions and Operators for anyURI
#

define function signature  fn:resolve-uri(xsd:anyURI, xsd:anyURI) as (xsd:anyURI)
define function signature  op:anyURI-equal(xsd:anyURI, xsd:anyURI) as (xsd:boolean)
define function signature  op:anyURI-nequal(xsd:anyURI, xsd:anyURI) as (xsd:boolean)


#
# F&O Section 11. Functions and Operators on base64Binary and hexBinary
#
# define function signature  op:hexBinary-equal(xsd:hexBinary, xsd:hexBinary) as (xsd:boolean)
# define function signature  op:base64Binary-equal(xsd:base64Binary, xsd:base64Binary) as (xsd:boolean)


#
# F&O Section 12. Functions and Operators on NOTATION
#
# define function signature  op:NOTATION-equal(xsd:NOTATION, xsd:NOTATION) as (xsd:boolean)


#
# F&O Section 13. Functions and Operators on Nodes
#

define function signature  fn:name(node?)                   as (xsd:string)
define function signature  fn:local-name(node?)             as (xsd:string)
define function signature  fn:namespace-uri(node?)          as (xsd:string)
define function signature  fn:lang(node,xs:string)          as (xsd:boolean)
define function signature  op:node-equal(node, node)        as (xsd:boolean)
define function signature  fn:deep-equal(node, node)        as (xsd:boolean)
define function signature  op:node-before(node, node)       as (xsd:boolean)
define function signature  op:node-after(node, node)        as (xsd:boolean)
define function signature  fn:copy(node)                    as (node)
define function signature  fn:root(node)                    as (node)


#
# F&O Section 14. Functions and Operators on Sequences
#

define function signature  op:to(xsd:int, xsd:int)          as (xsd:int+)

define function signature  fn:boolean(item*)                as (xsd:boolean)
define function signature  fn:item-at(item*, xsd:int)       as (item?)
define function signature  fn:index-of(node*, node)         as (xsd:int?)
define function signature  fn:empty(item*)            	    as (xsd:boolean)
define function signature  fn:exists(item*)            	    as (xsd:boolean)
define function signature  fn:distinct-nodes(node*)         as (node*)
define function signature  fn:distinct-values(item*)  	    as (item*)
define function signature  fn:insert(item*, xsd:int, item*) as (item*)
define function signature  fn:remove(item*, xsd:int)               as (item*)
define function signature  fn:subsequence(item*, xsd:int)          as (item*)
define function signature  fn:subsequence(item*, xsd:int, xsd:int) as (item*)

define function signature  fn:sequence-deep-equal(item*, item*) as (xs:boolean)
define function signature  fn:sequence-node-equal(item*, item*) as (xs:boolean)
define function signature  op:union(node*, node*)               as (node*)
define function signature  op:intersect(node*, node*)           as (node*)
define function signature  op:except(node*, node*)              as (node*)

define function signature  fn:count(item*)                      as (xsd:int)

#Note: aggregate functions are monomorphic in Galax for now...
define function signature  fn:avg(numeric*)	              		    as (numeric)
define function signature  fn:max(numeric*)	              		    as (numeric)
define function signature  fn:min(numeric*)	              		    as (numeric)
define function signature  fn:sum(numeric*)	              		    as (numeric)

# xsd:int
define function signature  fn:avg-int(xsd:int*)	              		    as (xsd:int)
define function signature  fn:max-int(xsd:int*)	              		    as (xsd:int?)
define function signature  fn:min-int(xsd:int*)	              		    as (xsd:int?)
define function signature  fn:sum-int(xsd:int*)	              		    as (xsd:int)
	      
# xsd:integer
define function signature  fn:avg-integer(xsd:integer*)			    as (xsd:integer)
define function signature  fn:max-integer(xsd:integer*)			    as (xsd:integer?)
define function signature  fn:min-integer(xsd:integer*)	              	    as (xsd:integer?)
define function signature  fn:sum-integer(xsd:integer*)	              	    as (xsd:integer)
	      
# xsd:decimal
define function signature  fn:avg-decimal(xsd:decimal*)	              	    as (xsd:decimal)
define function signature  fn:max-decimal(xsd:decimal*)	              	    as (xsd:decimal?)
define function signature  fn:min-decimal(xsd:decimal*)	              	    as (xsd:decimal?)
define function signature  fn:sum-decimal(xsd:decimal*)	              	    as (xsd:decimal)
	      
# xsd:float
define function signature  fn:avg-float(xsd:float*)	      		    as (xsd:float)
define function signature  fn:max-float(xsd:float*)	      		    as (xsd:float?)
define function signature  fn:min-float(xsd:float*)	      		    as (xsd:float?)
define function signature  fn:sum-float(xsd:float*)	      		    as (xsd:float)

# xsd:double
define function signature  fn:avg-double(xsd:double*)	      		    as (xsd:double)
define function signature  fn:max-double(xsd:double*)	      		    as (xsd:double?)
define function signature  fn:min-double(xsd:double*)	      		    as (xsd:double?)
define function signature  fn:sum-double(xsd:double*)	      		    as (xsd:double)

# Note: input functions are not quite right yet... 
define function signature  fn:document(xsd:string)                          as (document)


########################
# Comparison functions #
########################

# Note: Those functions are done in a polymorphic fashion in
# Galax. This differs from the F&O document where comparison functions
# are specialized by type.

# comparators

define function signature  op:equal(atomic, atomic)  as (xsd:boolean)
define function signature  op:nequal(atomic, atomic) as (xsd:boolean)
define function signature  op:ge(atomic, atomic)     as (xsd:boolean)
define function signature  op:le(atomic, atomic)     as (xsd:boolean)
define function signature  op:gt(atomic, atomic)     as (xsd:boolean)
define function signature  op:lt(atomic, atomic)     as (xsd:boolean)

###################
# Galax functions #
###################

# Note: Those functions are not part of the F&O spec. They are added
# in Galax for user's convenience.

# Sort by document order and duplicate removal

define function signature  glx:docorder(node*)                as (node*)
define function signature  glx:distinct-docorder(node*)       as (node*)

# "Type relief" operations				      
							      
define function signature  glx:unique(item*)                  as (item)
define function signature  glx:optional(item*)                as (item?)

# Union, intersection and exception on values

define function signature  glx:union-values(item*, item*)     as (item*)
define function signature  glx:intersect-values(item*, item*) as (item*)
define function signature  glx:except-values(item*, item*)    as (item*)

# Other input functions

define function signature  glx:document-sax(xsd:string)                      as (document)
define function signature  glx:document-validate(xsd:string, xsd:string)     as (document)

# Ouput functions

define function signature  glx:print-string(xsd:string)      as (empty)
define function signature  glx:print-item(item)              as (empty)

# xml:lang functions

define function signature  glx:get-lang(node)                as (xsd:string?)

